(* Auxiliary definitions, global variables, and datatypes *)

structure Data = struct 

exception Environment

exception Impossible of string

datatype baseval = 
    Int of int
  | Bool of bool

datatype baseop = 
    III of int * int -> int
  | IIB of int * int -> int		        (* 0 = false, 1 = true  *)

type trim = int list                            (* index list           *)

datatype expr = 
    App of expr * int                           (* application          *)
  | Var of int                                  (* variable             *)
  | Lam of expr                                 (* lambda abstraction   *)
  | Let of (expr * trim) list * (expr * trim)   (* letrec bindings      *)
  | Con of int * int                            (* constructor          *)
  | Case of expr * ((expr * trim) list * trim)  (* case expression      *)
  | Ret                                         (* base value closure   *)
  | Cst of int                                  (* base constant        *)
  | Op2 of int * int -> int                     (* dyadic base operator *)
  | Mkbool                                      (* make Boolean constr. *)
  | Prtint                                      (* print integer from V *)
  | Prtstr of string                            (* print string         *)
; 
datatype sourceexpr = 
    AppS of sourceexpr * sourceexpr
  | VarS of string
  | LamS of string * sourceexpr
  | LetS of (string * sourceexpr) list * sourceexpr
  | LetrecS of (string * sourceexpr) list * sourceexpr
  | ConS of int * sourceexpr list
  | CaseS of sourceexpr * alt list
  | CstS of baseval                             
  | Op2S of baseop * sourceexpr * sourceexpr
  | IfS  of sourceexpr * sourceexpr * sourceexpr
withtype alt = int * string list * sourceexpr

end
